@@ -1,8 +1,12 @@
 import re
 from abc import ABC, abstractmethod
+from collections import defaultdict
 from functools import wraps
 from pathlib import Path
-from typing import Any, Callable, List, Mapping, Optional, Set
+from typing import Any, Callable, DefaultDict, Dict, List, Optional, Set
+
+from jinja2 import Environment, FileSystemLoader, Template
+from pydantic import BaseModel, validator
 
 from datamodel_code_generator.imports import (
     IMPORT_LIST,
@@ -11,8 +15,6 @@ from datamodel_code_generator.imports import (
     Import,
 )
 from datamodel_code_generator.types import DataType, Types
-from jinja2 import Template
-from pydantic import BaseModel, validator
 
 TEMPLATE_DIR: Path = Path(__file__).parents[0] / 'template'
 
@@ -37,17 +39,20 @@ def optional(func: Callable) -> Callable:
 
 class DataModelField(BaseModel):
     name: Optional[str]
-    default: Optional[str]
+    default: Optional[Any]
     required: bool = False
     alias: Optional[str]
-    example: Optional[str]
+    example: Any
+    examples: Any
     description: Optional[str]
+    title: Optional[str]
     data_types: List[DataType] = []
     is_list: bool = False
     is_union: bool = False
     imports: List[Import] = []
     type_hint: Optional[str] = None
     unresolved_types: List[str] = []
+    field: Optional[str]
 
     @optional
     def _get_type_hint(self) -> Optional[str]:
@@ -77,8 +82,6 @@ class DataModelField(BaseModel):
 
     def __init__(self, **values: Any) -> None:
         super().__init__(**values)
-        if not self.alias and 'name' in values:
-            self.alias = values['name']
         for data_type in self.data_types:
             self.unresolved_types.extend(data_type.unresolved_types)
             if data_type.imports_:
@@ -89,8 +92,10 @@ class DataModelField(BaseModel):
 class TemplateBase(ABC):
     def __init__(self, template_file_path: Path) -> None:
         self.template_file_path: Path = template_file_path
-        self._template: Template = Template(
-            (TEMPLATE_DIR / self.template_file_path).read_text()
+        loader = FileSystemLoader(str(TEMPLATE_DIR / template_file_path.parent))
+        self.environment: Environment = Environment(loader=loader)
+        self._template: Template = self.environment.get_template(
+            template_file_path.name
         )
 
     @property
@@ -108,6 +113,10 @@ class TemplateBase(ABC):
         return self.render()
 
 
+class Config(BaseModel):
+    extra: Optional[str] = None
+
+
 class DataModel(TemplateBase, ABC):
     TEMPLATE_FILE_PATH: str = ''
     BASE_CLASS: str = ''
@@ -120,7 +129,7 @@ class DataModel(TemplateBase, ABC):
         base_classes: Optional[List[str]] = None,
         custom_base_class: Optional[str] = None,
         custom_template_dir: Optional[Path] = None,
-        extra_template_data: Optional[Mapping[str, Any]] = None,
+        extra_template_data: Optional[DefaultDict[str, Dict]] = None,
         imports: Optional[List[Import]] = None,
         auto_import: bool = True,
         reference_classes: Optional[List[str]] = None,
@@ -172,9 +181,9 @@ class DataModel(TemplateBase, ABC):
         self.class_name: str = class_name
 
         self.extra_template_data = (
-            extra_template_data.get(self.name, {})
+            extra_template_data[self.name]
             if extra_template_data is not None
-            else {}
+            else defaultdict(dict)
         )
 
         unresolved_types: Set[str] = set()
